@@ -102,7 +102,11 @@ class RoleMappingDecorator(object):
             # (This would cause problems if other references to the class
             # definition exist elsewhere.)
             new_meta = type('Meta', (subservice_class.Meta, ), { 'role_mapping': self.create_role_mapping() })
-            return type(subservice_class.__name__, (subservice_class, ), { 'Meta': new_meta })
+            return type(subservice_class.__name__, (subservice_class, ), {
+                        'Meta': new_meta,
+                        # Keep the module, to make sure that inspect.getsourcelines still works.
+                        '__module__': subservice_class.__module__,
+                        })
 
         # Else, passing in role mappings
         elif len(args) == 0 and len(kwargs):
@@ -272,6 +276,51 @@ class ServiceBase(type):
     # increased after every definition of a Service class.
     creation_counter = 0
 
+    @classmethod
+    def _preprocess_attributes(cls, attrs, base):
+        """
+        Do double-underscore preprocessing of attributes.
+        e.g.
+        `server__ssl_is_enabled = True` will override the `ssl_is_enabled`
+        value of the server object in attrs.
+        """
+        new_attrs = { }
+        override = { } # { attr_to_override: { k->v } }
+
+        # Split attributes in real attributes and "nested overrides".
+        for k,v in attrs.items():
+            if '__' in k and not k.startswith('__'): # Allow name mangling.
+                # Split at __ (only split at the first __, type(...) below
+                # does it recursively.)
+                attr_to_override, key = k.split('__', 1)
+
+                if attr_to_override in override:
+                    override[attr_to_override][key] = v
+                else:
+                    override[attr_to_override] = { key : v }
+            else:
+                new_attrs[k] = v
+
+        # Now apply overrides.
+        for attr, overrides in override.items():
+            first_override = overrides.keys()[0]
+
+            if attr in new_attrs:
+                raise Exception("Don't override %s__%s property in the same scope." %
+                                (attr, first_override))
+            elif hasattr(base, attr):
+                original_service = getattr(base, attr)
+
+                if not issubclass(original_service, Service):
+                    raise Exception('Service override %s__%s is not applied on a Service class.' %
+                                    (attr, first_override))
+                else:
+                    new_attrs[attr] = type(attr, (original_service,), overrides)
+            else:
+                raise Exception("Couldn't find %s__%s to override." % (attr, first_override))
+
+        return new_attrs
+
     def __new__(cls, name, bases, attrs):
         # No multiple inheritance allowed.
         if len(bases) > 1:
@@ -281,6 +330,9 @@ class ServiceBase(type):
         # Default action, if one was defined is some of the bases, use that one.
         default_action = getattr(bases[0], '_default_action', None)
 
+        # Preprocess __ in attributes
+        attrs = cls._preprocess_attributes(attrs, bases[0])
+
         if name != 'Service':
             # Do not allow __init__ to be overriden
             if '__init__' in attrs:
@@ -1123,16 +1175,16 @@ class Service(object):
             node = node.parent
         return node
 
-    def get_actions(self):
+    def get_actions(self, include_private=False):
         """
         Yield all the available actions. Yields tuples of (name, Action callable.)
         (Except the private ones)
         """
         for name, member in inspect.getmembers(self):
-            if not name.startswith('_') and isinstance(member, Action) and not getattr(self, name).is_property:
+            if (include_private or not name.startswith('_')) and isinstance(member, Action) and not getattr(self, name).is_property:
                 yield name, getattr(self, name)
 
-    def get_subservices(self, include_isolations=True):
+    def get_subservices(self, include_isolations=True, include_private=False):
         """
         Yield the available nested subservices. Returns tupels of (name, subservice_instance)
         (Except the private ones)
@@ -1141,7 +1193,7 @@ class Service(object):
         """
         # Sub services
         for name, member in inspect.getmembers(self):
-            if not name.startswith('_') and name != 'parent':
+            if (include_private or not name.startswith('_')) and name not in ('parent', '_creator_service'):
                 if isinstance(member, Service):
                     yield name, getattr(self, name)
 
@@ -1270,7 +1322,10 @@ def isolate_role(role):
     def isolate_decorator(service):
         # Don't modify the class, but create a new definition instead.
         new_meta = type('Meta', (service.Meta, ), { 'isolate_role': role })
-        return type(service.__name__, (service,), { 'Meta': new_meta })
+        return type(service.__name__, (service,), {
+                    'Meta': new_meta,
+                    '__module__': service.__module__,
+                     })
 
     # Add doc
     isolate_decorator.__doc__ = 'Turn on host isolation for the role "%s"' % role
