@@ -82,7 +82,7 @@ def create_navigable_handler(call_handler):
             if parent and name == '..':
                 return PathHandler(self.shell, parent)
 
-            for n, s in self.service.get_subservices():
+            for n, s in self.service.get_subservices(include_private=True):
                 if name == n:
                     return PathHandler(self.shell, s)
 
@@ -102,11 +102,12 @@ def create_navigable_handler(call_handler):
             if parent and '..'.startswith(part):
                 yield ('..', PathHandler(self.shell, parent))
 
-            for name, s in self.service.get_subservices():
+            # Note: when an underscore has been typed, include private too.
+            include_private = part.startswith('_')
+            for name, s in self.service.get_subservices(include_private=include_private):
                 if name.startswith(part):
                     yield name, PathHandler(self.shell, s)
 
-
         __call__ = call_handler
 
     class RootHandler(PathHandler):
@@ -154,57 +155,6 @@ class Version(ShellHandler):
         print termcolor.colored('  <%s>' % self.shell.root_service.__class__.__name__, 'red')
 
 
-
-class Do(ShellHandler):
-    is_leaf = False
-    sandbox = False
-    handler_type = BuiltinType()
-
-    def complete_subhandlers(self, part):
-        service = self.shell.state._service
-
-        # Root
-        if '/'.startswith(part):
-            yield '/', Service(find_root_service(service), self.shell, self.sandbox)
-
-        # Parent
-        if service.parent and '..'.startswith(part):
-            yield '..', Service(service.parent, self.shell, self.sandbox)
-
-        # Services
-        for name, s in service.get_subservices():
-            if name.startswith(part):
-                yield name, Service(s, self.shell, self.sandbox)
-
-        # Actions
-        for name, a in service.get_actions():
-            if name.startswith(part):
-                yield name, Action(a, self.shell, self.sandbox)
-
-    def get_subhandler(self, name):
-        service = self.shell.state._service
-
-        # Root
-        if name == '/':
-            return Service(find_root_service(service), self.shell, self.sandbox)
-
-        # Parent
-        if service.parent and name == '..':
-            return Service(service.parent, self.shell, self.sandbox)
-
-        # Services
-        subservices = dict(service.get_subservices())
-        if name in subservices:
-            service = subservices[name]
-            return Service(service, self.shell, self.sandbox)
-
-        # Actions
-        actions = dict(service.get_actions())
-        if name in actions:
-            action = actions[name]
-            return Action(action, self.shell, self.sandbox)
-
-
 @create_navigable_handler
 def Connect(self):
     """
@@ -352,6 +302,48 @@ def Ls(self):
 
     lesspipe(run(), self.shell.pty)
 
+@create_navigable_handler
+def SourceCode(self):
+    """
+    Print the source code of a service.
+    """
+    import inspect
+
+    from pygments import highlight
+    from pygments.lexers import PythonLexer
+    from pygments.formatters import TerminalFormatter
+    from deployer.console import choice, NoInput
+
+    options = []
+
+    for m in self.service.__class__.__mro__:
+        if m.__module__ != 'deployer.service' and m != object:
+            options.append( ('%s.%s' % (
+                  termcolor.colored(m.__module__, 'red'),
+                  termcolor.colored(m.__name__, 'yellow')), m) )
+
+    if len(options) > 1:
+        try:
+            service_class = choice('Choose service definition', options)
+        except NoInput:
+            return
+    else:
+        service_class = options[0][1]
+
+    def run():
+        try:
+            # Retrieve source
+            source = inspect.getsource(service_class)
+
+            # Highlight code
+            source = highlight(source, PythonLexer(), TerminalFormatter())
+
+            for l in source.split('\n'):
+                yield l.rstrip('\n')
+        except IOError:
+            yield 'Could not retrieve source code.'
+
+    lesspipe(run(), self.shell.pty)
 
 class Exit(ShellHandler):
     """
@@ -387,14 +379,6 @@ class Clear(ShellHandler):
         sys.stdout.flush()
 
 
-class Sandbox(Do):
-    """
-    Run commands in sandboxed mode. No code will be executed on remote servers.
-    """
-    sandbox = True
-    handler_type = BuiltinType()
-
-
 class Service(Handler):
     """
     Service node.
@@ -417,14 +401,20 @@ class Service(Handler):
             return type_of_service(self.service)
 
     def complete_subhandlers(self, part):
+        include_private = part.startswith('_')
+
         if self.service.parent and '..'.startswith(part):
             yield '..', Service(self.service.parent, self.shell, self.sandbox)
 
-        for name, action in self.service.get_actions():
+        if '/'.startswith(part):
+            root = find_root_service(self.service)
+            yield '/', Service(root, self.shell, self.sandbox)
+
+        for name, action in self.service.get_actions(include_private=include_private):
             if name.startswith(part):
                 yield name, Action(action, self.shell, self.sandbox)
 
-        for name, subservice in self.service.get_subservices():
+        for name, subservice in self.service.get_subservices(include_private=include_private):
             if name.startswith(part):
                 yield name, Service(subservice, self.shell, self.sandbox)
 
@@ -435,6 +425,9 @@ class Service(Handler):
         if name == '..' and self.service.parent:
             return Service(self.service.parent, self.shell, self.sandbox)
 
+        elif name == '/':
+            return Service(find_root_service(self.service), self.shell, self.sandbox)
+
         elif self.service.has_subservice(name):
             subservice = self.service.get_subservice(name)
             return Service(subservice, self.shell, self.sandbox)
@@ -450,6 +443,13 @@ class Service(Handler):
             return Action(self.service.get_action(self.service._default_action), self.shell, self.sandbox).__call__()
 
 
+class Sandbox(Service):
+    handler_type = BuiltinType()
+
+    def __init__(self, service, shell):
+        Service.__init__(self, service, shell, True)
+
+
 class Action(Handler):
     """
     Service action node.
@@ -490,41 +490,6 @@ class Action(Handler):
         # Report action call to logger interface
         action_callback = logger_interface.log_cli_action(command, sandbox)
 
-        def print_exec_failed_exception(e):
-            # hosts.run/sudo failed? Print error information.
-            print
-            print termcolor.colored('FAILED !!', 'red', attrs=['bold'])
-            print termcolor.colored('Command:     ', 'yellow'),
-            print termcolor.colored(e.command, 'red', attrs=['bold'])
-            print termcolor.colored('Host:        ', 'yellow'),
-            print termcolor.colored(e.host.slug, 'red', attrs=['bold'])
-            print termcolor.colored('Status code: ', 'yellow'),
-            print termcolor.colored(str(e.status_code), 'red', attrs=['bold'])
-            print
-
-        def print_query_exception(e):
-            print
-            print termcolor.colored('FAILED TO EXECUTE QUERY', 'red', attrs=['bold'])
-            print termcolor.colored('Service:     ', 'yellow'),
-            print termcolor.colored(e.service.__repr__(path_only=True), 'red', attrs=['bold'])
-            print termcolor.colored('Attribute:   ', 'yellow'),
-            print termcolor.colored(e.attr_name, 'red', attrs=['bold'])
-            print termcolor.colored('Query:       ', 'yellow'),
-            print termcolor.colored(e.query, 'red', attrs=['bold'])
-            print
-
-            if e.inner_exception:
-                if isinstance(e.inner_exception, ExecCommandFailed):
-                    print_exec_failed_exception(e.inner_exception)
-                else:
-                    print_other_exception(e.inner_exception)
-
-        def print_other_exception(e):
-            # Normal exception: print exception
-            print
-            print e
-            print
-
         try:
             if sandbox:
                 result = self.action(*self.args, _trace_action=True).sandbox(pty, logger_interface)
@@ -549,40 +514,11 @@ class Action(Handler):
                     handle_result(r.result)
 
         except ActionException, e:
-            if isinstance(e.inner_exception, ExecCommandFailed):
-                print_exec_failed_exception(e.inner_exception)
-
-            elif isinstance(e.inner_exception, QueryException):
-                print_query_exception(e.inner_exception)
-
-            else:
-                print '-'*79
-                print highlight(e.traceback, PythonTracebackLexer(), Formatter())
-                print '-'*79
-
-      #      # Print traceback through deployer services
-      #      print 'TODO: following trace is not entirely correct. It may show more deeper '
-      #      print '      than where the error actually occured.'
-
-      #      t = e.trace
-      #      while t:
-      #          from deployer.loggers.trace import TraceGroup
-      #          from deployer.loggers import Actions
-      #          if isinstance(t, TraceGroup):
-      #              print '- ', t.func_name
-      #              t = t.items[0] if t.items else None
-      #          elif t.entry_type == Actions.Run:
-      #              print '- (command) ', t.command
-      #              t = None
-      #          elif t.entry_type in (Actions.Put, Actions.Get, Actions.Open):
-      #              print '- (file) ', t.remote_path
-      #              t = None
-
-            action_callback.set_failed(e.inner_exception, e.traceback)
+            action_callback.set_failed(e)
 
         except Exception, e:
             # Print traceback and return to shell
-            print str(e)
+            print repr(e)
             print traceback.format_exc()
             action_callback.set_failed(e, traceback.format_exc())
 
@@ -609,7 +545,6 @@ class Action(Handler):
 
 class RootHandler(ShellHandler):
     subhandlers = {
-            '.': Do,
             'cd': Cd,
             'clear': Clear,
             'exit': Exit,
@@ -617,9 +552,18 @@ class RootHandler(ShellHandler):
             'ls': Ls,
             '--connect': Connect,
             '--inspect': Inspect,
-            '--sandbox': Sandbox,
             '--version': Version,
+            '--source-code': SourceCode,
     }
+
+    @property
+    def current_service(self):
+        return Service(self.shell.state._service, self.shell, sandbox=False)
+
+    @property
+    def sandboxed_current_service(self):
+        return Sandbox(self.shell.state._service, self.shell)
+
     def complete_subhandlers(self, part):
         """
         Return (name, Handler) subhandler pairs.
@@ -638,11 +582,23 @@ class RootHandler(ShellHandler):
             if name.startswith(part):
                 yield name, h(self.shell)
 
+        # Sandbox
+        if 'sandbox'.startswith(part):
+            yield 'sandbox', self.sandboxed_current_service
+
         # Services autocomplete for 'Do' -> 'do' is optional.
-        for name, h in Do(self.shell).complete_subhandlers(part):
+        dot = self.current_service
+        if '.'.startswith(part):
+            yield '.', dot
+
+        for name, h in dot.complete_subhandlers(part):
             yield name, h
 
     def get_subhandler(self, name):
+        # Current service
+        if name == '.':
+            return self.current_service
+
         # Default built-ins
         if name in self.subhandlers:
             return self.subhandlers[name](self.shell)
@@ -650,12 +606,16 @@ class RootHandler(ShellHandler):
         if self.shell.state.can_return and name == 'return':
             return Return(self.shell)
 
+        if name == 'sandbox':
+            return self.sandboxed_current_service
+
         # Extensions
         if name in self.shell.extensions:
             return self.shell.extensions[name](self.shell)
 
         # Services autocomplete for 'Do' -> 'do' is optional.
-        return Do(self.shell).get_subhandler(name)
+        dot = self.current_service
+        return dot.get_subhandler(name)
 
 
 class ShellState(object):
